/*
 * Copyright (c) 2013-2019 Huawei Technologies Co., Ltd. All rights reserved.
 * Copyright (c) 2020-2021 Huawei Device Co., Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "It_posix_pthread.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cpluscplus */
#endif /* __cpluscplus */

static VOID *PthreadF01(void *argument)
{
    g_testCount++;

    return argument;
}
static UINT32 Testcase(VOID)
{
    pthread_attr_t attr;
    pthread_t newTh;
    UINT32 ret;
    UINTPTR temp;
    int policy;
    struct sched_param param;
    struct sched_param param2 = { 2 }; // 2, init

    g_testCount = 0;

    ret = pthread_attr_init(&attr);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);

    ret = pthread_create(&newTh, NULL, PthreadF01, (void *)9); // 9, test param of the function.
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);
    LOS_TaskDelay(1);
    ICUNIT_ASSERT_EQUAL(g_testCount, 1, g_testCount);

    ret = pthread_setschedparam(newTh, 0, &param);
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);

    ret = pthread_setschedparam(newTh, 4, &param); // 4, test for invaild param.
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);

    ret = pthread_setschedparam(newTh, SCHED_RR, NULL);
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);

    param.sched_priority = 0;
    ret = pthread_setschedparam(newTh, SCHED_RR, &param);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);

    param.sched_priority = 31; // 31, init
    ret = pthread_setschedparam(newTh, SCHED_RR, &param);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);

    param.sched_priority = 32; // 32, init
    ret = pthread_setschedparam(newTh, SCHED_RR, &param);
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);

    ret = pthread_getschedparam(newTh, NULL, &param2);
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);
    ICUNIT_ASSERT_EQUAL(param2.sched_priority, 2, param2.sched_priority); // 2, here assert the result.

    ret = pthread_getschedparam(newTh, &policy, NULL);
    ICUNIT_ASSERT_EQUAL(ret, EINVAL, ret);

    ret = pthread_getschedparam(newTh, &policy, &param2);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);
    ICUNIT_ASSERT_EQUAL(param2.sched_priority, 31, param2.sched_priority); // 31, here assert the result.

    ret = pthread_join(newTh, (void *)&temp);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);
    ICUNIT_ASSERT_EQUAL(temp, 9, temp); // 9, here assert the result.

    param.sched_priority = 4; // 4, init
    ret = pthread_setschedparam(newTh, SCHED_RR, &param);
    ICUNIT_ASSERT_EQUAL(ret, ESRCH, ret);

    ret = pthread_setschedparam(newTh + 9, SCHED_RR, &param); // 9, test for invaild param.
    ICUNIT_ASSERT_EQUAL(ret, ESRCH, ret);

    ret = pthread_getschedparam(newTh, &policy, &param2);
    ICUNIT_ASSERT_EQUAL(ret, ESRCH, ret);

    ret = pthread_getschedparam(newTh + 8, &policy, &param2); // 8, test for invaild param.
    ICUNIT_ASSERT_EQUAL(ret, ESRCH, ret);

    ret = pthread_attr_destroy(&attr);
    ICUNIT_ASSERT_EQUAL(ret, 0, ret);
    return PTHREAD_NO_ERROR;
}

VOID ItPosixPthread018(VOID) // IT_Layer_ModuleORFeature_No
{
    TEST_ADD_CASE("ItPosixPthread018", Testcase, TEST_POSIX, TEST_PTHREAD, TEST_LEVEL0, TEST_FUNCTION);
}

#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cpluscplus */
#endif /* __cpluscplus */